==> em_tasks-0.1.8-py2.py3-none-any.whl: em_tasks/tasks/spapi_update_catalog_items_task.py <==
# -*- coding: utf-8 -*-

import time
import datetime

import dateutil
import dateutil.parser

from pint import UnitRegistry

from em_tasks import logger


class SpapiUpdateCatalogItemsTask():
  def __init__(self, spapi, product_service, marketplace, asins):
    self.spapi = spapi
    self.product_service = product_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.ureg = UnitRegistry()
    self.dimentions_name = ['weight', 'width', 'length', 'height']
    self.dimention_units = {
      'weight': 'lbs',
      'width': 'inch',
      'length': 'inch',
      'height': 'inch',
    }
    self.product_flat_attrs = [
      'asin', 'attributes', 'dimensions', 'identifiers', 'productTypes',
      'relationships', 'salesRanks', 'summaries'
    ]
    self.has_value_array_attrs_in_summary = ['edition', 'manufacturer', 'publication_date']
    self.has_value_array_attrs_in_attributes = ['binding', 'pages']
    self.array_attrs_in_attributes = ['item_dimensions', 'item_package_dimensions']

  def run(self):
    cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
    indice_name = 'amz_products_api_{}'.format(self.marketplace)

    products_info = self.search_catalog_items(self.spapi, self.marketplace, self.asins)
    if not products_info:
      return

    for _, product_info in products_info.items():
      product_info['_id'] = product_info['asin']
      product_info['time'] = cur_time
    try:
      self.product_service.save_products(indice_name, list(products_info.values()))
    except Exception as e:
      logger.warning('[ProductSaveToServiceError] %s', products_info)
      logger.exception(e)

  def search_catalog_items(self, spapi, marketplace, asins):
    response = spapi.search_catalog_items(asins, marketplace=marketplace)
    if not response:
      return

    d = dict()
    product_dicts = {}
    parsed_resp = response.payload
    results = parsed_resp['items']
    for product_dict in results:
      if "Error" in product_dict:
        logger.warning(
          '[SPAPIError] %s', product_dict['Error'].get('Message', d).get('value', ''))
        continue

      product = {}

      for attr in self.product_flat_attrs:
        product[attr] = product_dict.get(attr)

        images = []
        for img in product_dict['images'][0]['images']:
          images.append(img['link'])

        try:
          title = product.get('attributes', {}).get('item_name', [{}])[0].get('value', '')[:255]
          brand = product.get('attributes', {}).get('brand', [{}])[0].get('value', '')[:100]
          format = ""

          try:
            sales_ranking = int(product.get('salesRanks', [{}])[0]
              .get('displayGroupRanks', [{}])[0].get('rank', 0))
          except:
            sales_ranking = 0

          dimension = {}
          dimensions = product['dimensions']
          lwh = 0
          if isinstance(dimensions, list) and len(dimensions) > 0 and isinstance(dimensions[0], dict):
            di = dimensions[0]
            dimension = {
              dimension_name: self.get_dimension_value(di, dimension_name)
              for dimension_name in self.dimention_units
            }

            weight = dimension.pop('weight', 0)
            for dimension_key in dimension:
              lwh += int(float(dimension.get(dimension_key, 0)))

          ranks = []
          rank_types = ['classificationRanks', 'displayGroupRanks']
          for rank in product.get('salesRanks', []):
            for rank_type in rank_types:
              for rank_dict in rank.get(rank_type, []):
                ranks.append({
                  'category': rank_dict.get('title', ''),
                  'rank': rank_dict.get('rank', '')
                })

          asin = product['asin']
          product_dicts[asin] = {
            'title': title,
            'brand': brand,
            'weight': weight,
            'format': format,
            'sales_rank': sales_ranking,
            'lwh': lwh,
            'sales_ranks': ranks,
            'images': images,
            **product
          }
          attributes = product.get('attributes', {})

          for attr in self.has_value_array_attrs_in_attributes:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0].get('value', '')

          for attr in self.array_attrs_in_attributes:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0]

          product_dicts[asin]['item_weight'] = self.get_weight_from_attributes('item_weight', attributes)
          product_dicts[asin]['item_package_weight'] = self.get_weight_from_attributes('item_package_weight', attributes)

          if 'subject_keyword' in attributes:
            product_dicts[asin]['subject_keyword'] = [subject['value'] for subject in attributes['subject_keyword']]

          if 'generic_keyword' in attributes:
            product_dicts[asin]['generic_keyword'] = [subject['value'] for subject in attributes['generic_keyword']]

          product_dicts[asin]['item_dimensions'] = attributes.get('item_dimensions', [{}])[0]

          summaries = product_dicts.get('summaries', {})
          for attr in self.has_value_array_attrs_in_summary:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0].get('value', '')

          if 'list_price' in summaries:
            product_dicts[asin]['list_price'] = {'currency': attributes['list_price'][0]['currency'], 'value': attributes['list_price'][0]['value']}
        except Exception as e:
          logger.exception(e)

    return product_dicts

  def get_dimension_value(self, dimension_obj, dimension_name):
    packaege_or_item_dimension = dimension_obj.get('package', dimension_obj.get('item', {})).get(dimension_name, {})
    if not packaege_or_item_dimension.get('value'):
      return 0

    value_with_unit = '{} {}'.format(packaege_or_item_dimension.get('value', 0), packaege_or_item_dimension.get('unit', ''))
    return round(self.ureg(value_with_unit).to(self.dimention_units.get(dimension_name)).m, 2)

  def get_weight_from_attributes(self, attr, attributes):
    weight = attributes.get(attr, [{}])[0]
    if not weight.get('value'):
      return 0

    weight_value_with_unit = '{} {}'.format(weight.get('value', 0), weight.get('unit', 'pound'))
    return round(self.ureg(weight_value_with_unit).to(self.dimention_units.get('weight')).m, 2)

==> em_tasks-0.1.19-py2.py3-none-any.whl: em_tasks/tasks/spapi_update_catalog_items_task.py <==
# -*- coding: utf-8 -*-

import time
import datetime

import dateutil
import dateutil.parser

from pint import UnitRegistry

from em_tasks import logger


class SpapiUpdateCatalogItemsTask():
  def __init__(self, spapi, product_service, marketplace, asins):
    self.spapi = spapi
    self.product_service = product_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.ureg = UnitRegistry()
    self.dimentions_name = ['weight', 'width', 'length', 'height']
    self.dimention_units = {
      'weight': 'lbs',
      'width': 'inch',
      'length': 'inch',
      'height': 'inch',
    }
    self.product_flat_attrs = [
      'asin', 'attributes', 'dimensions', 'identifiers', 'productTypes',
      'relationships', 'salesRanks', 'summaries'
    ]
    self.has_value_array_attrs_in_summary = ['edition', 'manufacturer', 'publication_date']
    self.has_value_array_attrs_in_attributes = ['binding', 'pages']
    self.array_attrs_in_attributes = ['item_dimensions', 'item_package_dimensions']

  def run(self):
    cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
    indice_name = 'amz_products_api_{}'.format(self.marketplace)

    products_info = self.search_catalog_items(self.spapi, self.marketplace, self.asins)
    if not products_info:
      return

    for _, product_info in products_info.items():
      product_info['_id'] = product_info['asin']
      product_info['time'] = cur_time
    try:
      self.product_service.save_products(indice_name, list(products_info.values()))
    except Exception as e:
      logger.warning('[ProductSaveToServiceError] %s', products_info)
      logger.exception(e)

  def search_catalog_items(self, spapi, marketplace, asins):
    response = spapi.search_catalog_items(asins, marketplace=marketplace)
    if not response:
      return

    d = dict()
    product_dicts = {}
    parsed_resp = response.payload
    results = parsed_resp['items']
    for product_dict in results:
      if "Error" in product_dict:
        logger.warning(
          '[SPAPIError] %s', product_dict['Error'].get('Message', d).get('value', ''))
        continue

      product = {}

      for attr in self.product_flat_attrs:
        product[attr] = product_dict.get(attr)

        images = []
        for img in product_dict['images'][0]['images']:
          images.append(img['link'])

        try:
          title = product.get('attributes', {}).get('item_name', [{}])[0].get('value', '')[:255]
          brand = product.get('attributes', {}).get('brand', [{}])[0].get('value', '')[:100]
          format = ""

          try:
            sales_ranking = int(product.get('salesRanks', [{}])[0]
              .get('displayGroupRanks', [{}])[0].get('rank', 0))
          except:
            sales_ranking = 0

          dimension = {}
          dimensions = product.get('dimensions', [])
          lwh = 0
          weight = None
          if isinstance(dimensions, list) and len(dimensions) > 0 and isinstance(dimensions[0], dict):
            di = dimensions[0]
            dimension = {
              dimension_name: self.get_dimension_value(di, dimension_name)
              for dimension_name in self.dimention_units
            }

            weight = dimension.pop('weight', 0)
            for dimension_key in dimension:
              lwh += int(float(dimension.get(dimension_key, 0)))

          ranks = []
          rank_types = ['classificationRanks', 'displayGroupRanks']
          for rank in product.get('salesRanks', []):
            for rank_type in rank_types:
              for rank_dict in rank.get(rank_type, []):
                ranks.append({
                  'category': rank_dict.get('title', ''),
                  'rank': rank_dict.get('rank', '')
                })

          asin = product['asin']
          product_dicts[asin] = {
            'title': title,
            'brand': brand,
            'weight': weight,
            'format': format,
            'sales_rank': sales_ranking,
            'lwh': lwh,
            'sales_ranks': ranks,
            'images': images,
            **product
          }
          attributes = product.get('attributes', {})

          for attr in self.has_value_array_attrs_in_attributes:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0].get('value', None)

          for attr in self.array_attrs_in_attributes:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0]

          product_dicts[asin]['item_weight'] = self.get_weight_from_attributes('item_weight', attributes)
          product_dicts[asin]['item_package_weight'] = self.get_weight_from_attributes('item_package_weight', attributes)

          if 'subject_keyword' in attributes:
            product_dicts[asin]['subject_keyword'] = [subject['value'] for subject in attributes['subject_keyword']]

          if 'generic_keyword' in attributes:
            product_dicts[asin]['generic_keyword'] = [subject['value'] for subject in attributes['generic_keyword']]

          product_dicts[asin]['item_dimensions'] = attributes.get('item_dimensions', [{}])[0]

          summaries = product_dicts.get('summaries', {})
          for attr in self.has_value_array_attrs_in_summary:
            product_dicts[asin][attr] = attributes.get(attr, [{}])[0].get('value', None)

          if 'list_price' in summaries:
            product_dicts[asin]['list_price'] = {'currency': attributes['list_price'][0]['currency'], 'value': attributes['list_price'][0]['value']}
        except Exception as e:
          logger.exception(e)

    return product_dicts

  def get_dimension_value(self, dimension_obj, dimension_name):
    packaege_or_item_dimension = dimension_obj.get('package', dimension_obj.get('item', {})).get(dimension_name, {})
    if not packaege_or_item_dimension.get('value'):
      return 0

    value_with_unit = '{} {}'.format(packaege_or_item_dimension.get('value', 0), packaege_or_item_dimension.get('unit', ''))
    return round(self.ureg(value_with_unit).to(self.dimention_units.get(dimension_name)).m, 2)

  def get_weight_from_attributes(self, attr, attributes):
    weight = attributes.get(attr, [{}])[0]
    if not weight.get('value'):
      return 0

    weight_value_with_unit = '{} {}'.format(weight.get('value', 0), weight.get('unit', 'pound'))
    return round(self.ureg(weight_value_with_unit).to(self.dimention_units.get('weight')).m, 2)
